# -*- coding: utf-8 -*-
from flask import Flask, render_template
from packagesample import __version__, log
from packagesample.modules import *
import __builtin__

app = Flask(__name__)
__builtin__.chicken = False


@app.context_processor
def get_title():
    return dict(get_title='packagesample v' + str(__version__))


@app.route("/")
def home():
    return render_template('start.html', text=module.main())


def main():
    app.run(host='0.0.0.0', port=5000, debug=True)



# -*- coding: UTF-8 -*-
# pep8: disable-msg=E501
# pylint: disable=C0301
import os
import logging
import getpass
import tempfile

__version__ = '0.0.1'
__author__ = 'Your Name'
__author_username__ = 'your_username'
__author_email__ = 'yourname@gmail.com'
__description__ = 'Generated from a template'
__chicken__ = False

log_filename = os.path.join(tempfile.gettempdir(),
                            'packagesample-' + getpass.getuser() + '.log')

log = logging
log.basicConfig(level=logging.DEBUG,
                format='%(asctime)s %(levelname)s %(message)s',
                filename=log_filename,
                filemode='a')


def __path(filename):
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), filename)

if os.path.exists(__path('build.info')):
    __build__ = open(__path('build.info')).read().strip()
else:
    __build__ = '0'

__version__ = __version__ + '.' + __build__



# -*- coding: utf-8 -*-
# pep8: disable-msg=E501
# pylint: disable=C0301

from packagesample import __version__, log, modules
import argparse
import __builtin__


def main():
    log.info("packagesample v" + __version__)
    parser = argparse.ArgumentParser(
        description='packagesample v' + __version__)

    parser.add_argument(
        '-x', '--xunxo', help='Sample NonRequired Parameter', required=False)

    tasks = modules.__all__

    try:
        if len(tasks) > 1:
            tasks_str = ', '.join(tasks)
            k = tasks_str.rfind(",")
            tasks_str = tasks_str[:k] + " or" + tasks_str[k + 1:]
        else:
            tasks_str = tasks[0]

        parser.add_argument(
            '-t', '--task',
            help='Task to be performed. Allowed values: ' + tasks_str + '.',
            metavar='<task_name>',
            choices=tasks,
            required=True)
    except:
        pass

    parser.add_argument(
        "--chicken",
        help="Chicken mode (optional). Does NOT CHANGE anything.",
        action="store_true")

    args = parser.parse_args()  # if required params arent met, program aborts
    __builtin__.chicken = args.chicken

    try:
        result = eval(args.task + '.main()')
        result = 'RESULT for "' + args.task + '" = ' + result
        log.debug(result)
        exit(0)
    except Exception as e:
        import traceback
        log.error(traceback.format_exc())
        exit(1)



# -*- coding: UTF-8 -*-
from machete import __version__, log, log_filename
import shutil
import errno
import os
import subprocess
import stat
import tempfile
IGNORE_PATTERNS = ('*.pyc')

is_chicken = False

windows = os.name == 'nt'


def os_call(path):
    log.debug(path)
    proc = subprocess.Popen(path.split(' '), stdout=subprocess.PIPE)
    (out, err) = proc.communicate()
    return out


def replace_infile(lookfor, replace, text_file):
    try:
        infile = open(text_file)
        outfile = open(text_file + '.tmp', 'w')

        # replacements = {'zero':'0', 'temp':'bob', 'garbage':'nothing'}
        replacements = {lookfor: replace}

        for line in infile:
            for src, target in replacements.iteritems():
                line = line.replace(src, target)
            outfile.write(line)
        infile.close()
        outfile.close()
        shutil.move(text_file + '.tmp', text_file)
        return True
    except Exception, e:
        print str(e)
        return False


def copytree(src, dst, ignore=shutil.ignore_patterns(IGNORE_PATTERNS)):
    # print('trying to copy from '+src+' to '+dst)
    if not os.path.exists(dst):
        os.makedirs(dst)
        shutil.copystat(src, dst)
    lst = os.listdir(src)
    if ignore:
        excl = ignore(src, lst)
        lst = [x for x in lst if x not in excl]
    for item in lst:
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            copytree(s, d, ignore)
        else:
            shutil.copy2(s, d)


def copy_files(template):
    print("Copying files from template...")
    try:
        if not is_chicken:
            run_file = os.path.realpath(__file__)
            run_folder = os.path.dirname(run_file)
            path = os.path.join(os.path.dirname(run_folder), "templates")

            base_path = os.path.join(path, "_base")
            template_path = os.path.join(path, template)

            copytree(base_path, '.')
            copytree(template_path, '.')
        return True
    except Exception, e:
        print str(e)
        return False


def git_init():
    print("Initializing GIT...")
    if not is_chicken:
        try:
            os_call('git init')
        except:
            pass  # thats ok, you can sort of live without git


def rename_files(project):
    print("Modifying copied files names...")
    try:
        if not is_chicken:
            shutil.move('packagesample', project)
            shutil.move('docs/example/packagesample.cfg',
                        'docs/example/' + project + '.cfg')
        return True
    except Exception, e:
        print str(e)
        return False


def perform_replaces(project):
    print("Modifying copied files content...")
    try:
        if not is_chicken:
            files = ['setup.py', 'README.rst', 'run.py', 'MANIFEST.in',
                     'docs/source/changelog.rst', project + '/start.py',
                     project + '/__init__.py', project + '/modules/module.py',
                     project + '/modules/__init__.py', 'tests/test_version.py',
                     'setup.cfg']
            for each in files:
                replace_infile('packagesample', project, each)
        return True
    except Exception, e:
        print str(e)
        return False


def has_virtualenv():
    try:
        return 'Usage' in os_call('virtualenv')
    except:
        return False


def has_virtualenvwrapper():
    if has_virtualenv():
        try:
            if not windows:
                return '/usr' in os_call('which virtualenvwrapper.sh')
            else:
                return False
        except:
            return False
    else:
        return False


def create_venv(project):
    print("Creating virtualenv...")
    try:
        if not is_chicken:
            log.debug('Trying to create virtualenv')
            env_path = os.getenv('WORKON_HOME', './.venv_')
            full_env_path = env_path
            if has_virtualenvwrapper():
                full_env_path = os.path.join(env_path, project)
                os_call('virtualenv --clear ' + full_env_path)
            else:
                if has_virtualenv():
                    full_env_path = os.path.join(env_path + project)
                    os_call('virtualenv ' + full_env_path)

            print("\nSUCCESS!!!")

            activateL = full_env_path + '/bin/activate'
            activateW = full_env_path + '/Scripts/activate'
            if windows:
                activate = activateW
            else:
                activate = activateL

            if os.path.isfile(activate):
                print("Please execute 'source " + activate +
                      "' to enter into your virtualenv")
                # http://stackoverflow.com/questions/6943208/activate-a-virtualenv-with-a-python-script

                if has_virtualenv():
                    if os.path.isfile('requirements.txt'):
                        print("Then please execute \
                              'pip install -r requirements.txt'")
                        # os_call('pip install -r requirements.txt')
                    if os.path.isfile('requirements-dev.txt'):
                        print("And after please execute \
                              'pip install -r requirements-dev.txt'")
                        # os_call('pip install -r requirements-dev.txt')
        return True
    except Exception, e:
        print str(e)
        return False


def main(template, chicken):
    global is_chicken
    is_chicken = chicken
    if is_chicken:
        log.debug('RUNNING ON CHICKEN MODE')
        print('*** RUNNING ON CHICKEN MODE ***')

    if os.listdir('.'):
        message = 'Current directory is not empty, so machete cannot run :-('
        log.warn(message)
        print(message)
        exit(1)

    project = os.path.basename(os.getcwd()).replace('-', '_')
    git_init()
    if copy_files(template):
        if rename_files(project):
            if perform_replaces(project):
                if create_venv(project):
                    print('\nmachete says: "Its done!"')
                    print('\nAfter these steps, run your \
                          app with "python run.py"')
                    print('Check for the log file under ' +
                          tempfile.gettempdir())



# -*- coding: UTF-8 -*-
import glob
import os

__all__ = []

modules_folder = os.path.dirname(os.path.abspath(__file__))
py_files = glob.glob(modules_folder + '/*.py')
py_files.remove(os.path.abspath(__file__).replace('.pyc', '.py'))

for py_file in py_files:
    py_file = './' + '/'.join(py_file.split('/')[-3:])
    __all__.append(py_file.split('/')[3].split('.')[0])
